# Copyright (c) 2021 Skyward Experimental Rocketry
# Authors: Damiano Amatruda
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

# Load in BOARDCORE_PATH the project path
cmake_path(GET CMAKE_CURRENT_LIST_DIR PARENT_PATH BOARDCORE_PATH)

# TODO: include the miosix_opt_board_list and boardcore_opt_board_list and dispatch automatically in base of this

# Include all the architecture and board specific defines
if(DEFINED BOARDCORE_OPT_BOARD)
    include(${BOARDCORE_PATH}/src/bsps/CMakeLists.txt)
#elseif(${MIOSIX_OPT_BOARD})
#    include(${BOARDCORE_PATH}/libs/miosix-kernel/miosix/arch/CMakeLists.txt)
endif()

# Include dependencies and board list
include(${BOARDCORE_PATH}/cmake/dependencies.cmake)
include(${BOARDCORE_PATH}/cmake/boardcore-host.cmake)
include(${BOARDCORE_PATH}/cmake/boards.cmake)

# Boardcore source files
set(BOARDCORE_SRC
    # Actuators
    ${BOARDCORE_PATH}/src/shared/actuators/HBridge/HBridge.cpp
    ${BOARDCORE_PATH}/src/shared/actuators/Servo/Servo.cpp
    ${BOARDCORE_PATH}/src/shared/actuators/stepper/Stepper.cpp
    ${BOARDCORE_PATH}/src/shared/actuators/stepper/StepperPWM.cpp

    # Algorithms
    # ${BOARDCORE_PATH}/src/shared/algorithms/ADA/ADA.cpp
    # ${BOARDCORE_PATH}/src/shared/algorithms/MEA/MEA.cpp
    ${BOARDCORE_PATH}/src/shared/algorithms/AirBrakes/AirBrakesPI.cpp
    ${BOARDCORE_PATH}/src/shared/algorithms/AirBrakes/AirBrakesInterp.cpp
    ${BOARDCORE_PATH}/src/shared/algorithms/NAS/NAS.cpp
    ${BOARDCORE_PATH}/src/shared/algorithms/NAS/StateInitializer.cpp
    ${BOARDCORE_PATH}/src/shared/algorithms/Propagator/Propagator.cpp
    ${BOARDCORE_PATH}/src/shared/algorithms/Follower/Follower.cpp

    # Debug
    ${BOARDCORE_PATH}/src/shared/utils/Debug.cpp
    ${BOARDCORE_PATH}/src/shared/diagnostic/CpuMeter/CpuMeter.cpp
    ${BOARDCORE_PATH}/src/shared/diagnostic/PrintLogger.cpp

    # Drivers
    ${BOARDCORE_PATH}/src/shared/drivers/AD5204/AD5204.cpp
    ${BOARDCORE_PATH}/src/shared/drivers/adc/InternalADC.cpp
    ${BOARDCORE_PATH}/src/shared/drivers/canbus/CanDriver/CanDriver.cpp
    ${BOARDCORE_PATH}/src/shared/drivers/canbus/CanDriver/CanInterrupt.cpp
    ${BOARDCORE_PATH}/src/shared/drivers/canbus/CanProtocol/CanProtocol.cpp
    ${BOARDCORE_PATH}/src/shared/drivers/interrupt/external_interrupts.cpp
    ${BOARDCORE_PATH}/src/shared/drivers/timer/PWM.cpp
    ${BOARDCORE_PATH}/src/shared/drivers/timer/CountedPWM.cpp
    ${BOARDCORE_PATH}/src/shared/drivers/runcam/Runcam.cpp
    ${BOARDCORE_PATH}/src/shared/drivers/spi/SPITransaction.cpp
    ${BOARDCORE_PATH}/src/shared/drivers/usart/USART.cpp
    ${BOARDCORE_PATH}/src/shared/drivers/i2c/I2CDriver-f4.cpp
    ${BOARDCORE_PATH}/src/shared/drivers/i2c/I2CDriver-f7.cpp
    ${BOARDCORE_PATH}/src/shared/drivers/i2c/I2C.cpp
    ${BOARDCORE_PATH}/src/shared/drivers/WIZ5500/WIZ5500.cpp
    ${BOARDCORE_PATH}/src/shared/drivers/DipSwitch/DipSwitch.cpp

    # Events
    ${BOARDCORE_PATH}/src/shared/events/EventBroker.cpp

    # Logger
    ${BOARDCORE_PATH}/src/shared/logger/Logger.cpp

    # Radio
    ${BOARDCORE_PATH}/src/shared/radio/gamma868/Gamma868.cpp
    ${BOARDCORE_PATH}/src/shared/radio/Xbee/APIFrameParser.cpp
    ${BOARDCORE_PATH}/src/shared/radio/Xbee/Xbee.cpp
    ${BOARDCORE_PATH}/src/shared/radio/SX1278/SX1278Fsk.cpp
    ${BOARDCORE_PATH}/src/shared/radio/SX1278/SX1278Lora.cpp
    ${BOARDCORE_PATH}/src/shared/radio/SX1278/SX1278Common.cpp

    # Scheduler
    ${BOARDCORE_PATH}/src/shared/scheduler/TaskScheduler.cpp

    # Sensors
    ${BOARDCORE_PATH}/src/shared/sensors/ADS1118/ADS1118.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/ADS131M04/ADS131M04.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/ADS131M08/ADS131M08.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/BME280/BME280.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/BME280/BME280I2C.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/BMP280/BMP280.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/BMP280/BMP280I2C.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/BMX160/BMX160.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/BMX160/BMX160WithCorrection.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/H3LIS331DL/H3LIS331DL.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/HX711/HX711.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/LIS3MDL/LIS3MDL.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/LIS331HH/LIS331HH.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/LPS331AP/LPS331AP.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/MAX6675/MAX6675.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/MAX31855/MAX31855.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/MAX31856/MAX31856.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/MBLoadCell/MBLoadCell.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/MPU9250/MPU9250.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/MS5803/MS5803.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/MS5803/MS5803I2C.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/RotatedIMU/RotatedIMU.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/SensorManager.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/SensorSampler.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/UBXGPS/UBXGPSSerial.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/UBXGPS/UBXGPSSpi.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/Vectornav/VNCommonSerial.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/Vectornav/VN100/VN100Serial.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/Vectornav/VN100/VN100Spi.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/Vectornav/VN300/VN300.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/LIS2MDL/LIS2MDL.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/LPS28DFW/LPS28DFW.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/LPS22DF/LPS22DF.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/LSM6DSRX/LSM6DSRX.cpp

    # Calibration
    ${BOARDCORE_PATH}/src/shared/sensors/calibration/BiasCalibration/BiasCalibration.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/calibration/SensorDataExtra/SensorDataExtra.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/calibration/SixParameterCalibration/SixParameterCalibration.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/calibration/SoftAndHardIronCalibration/SoftAndHardIronCalibration.cpp

    # Correction
    ${BOARDCORE_PATH}/src/shared/sensors/correction/BiasCorrector/BiasCorrector.cpp
    ${BOARDCORE_PATH}/src/shared/sensors/correction/SixParametersCorrector/SixParametersCorrector.cpp

    # Utils
    ${BOARDCORE_PATH}/src/shared/utils/AeroUtils/AeroUtils.cpp
    ${BOARDCORE_PATH}/src/shared/utils/ButtonHandler/ButtonHandler.cpp
    ${BOARDCORE_PATH}/src/shared/utils/PinObserver/PinObserver.cpp
    ${BOARDCORE_PATH}/src/shared/utils/SkyQuaternion/SkyQuaternion.cpp
    ${BOARDCORE_PATH}/src/shared/utils/Stats/Stats.cpp
    ${BOARDCORE_PATH}/src/shared/utils/TestUtils/TestHelper.cpp
    ${BOARDCORE_PATH}/src/shared/utils/Registry/RegistryFrontend.cpp
    ${BOARDCORE_PATH}/src/shared/utils/Registry/RegistrySerializer.cpp
    ${BOARDCORE_PATH}/src/shared/utils/Registry/Backend/FileBackend.cpp
    ${BOARDCORE_PATH}/src/shared/utils/DependencyManager/DependencyManager.cpp
)

set(PROTECT_SRC
    ${BOARDCORE_PATH}/src/shared/aspis/aspis.cpp
    ${BOARDCORE_PATH}/src/shared/algorithms/MEA/MEA.cpp
    ${BOARDCORE_PATH}/src/shared/algorithms/ADA/ADA.cpp
)

function(link_boardcore_submodules TARGET TYPE)
    # Only one include directory for Boardcore!
    target_include_directories(${TARGET} ${TYPE} ${BOARDCORE_PATH}/src/shared)

    # Define DEBUG when in Debug mode
    target_compile_definitions(${TARGET} ${TYPE} $<$<CONFIG:Debug>:DEBUG>)
    # Define NDEBUG when not in Debug mode
    target_compile_definitions(${TARGET} ${TYPE} $<$<NOT:$<CONFIG:Debug>>:NDEBUG>)

    # Link libraries
    include(${MIOSIX_KPATH}/cmake/LinkTarget.cmake)
    miosix_link_target(${TARGET} ${TYPE})

endfunction()

function(link_boardcore_libraries TARGET TYPE)
    target_link_libraries(${TARGET} ${TYPE}
        TSCPP::TSCPP
        Eigen3::Eigen
        fmt::fmt-header-only
        Catch2::Catch2
        Mavlink::Mavlink
    )

    # Link MxGui if supported by the target
    if(DEFINED MXGUI_BASE_BOARD_NAME)
        target_link_libraries(${TARGET} ${TYPE} MxGui::${MXGUI_BASE_BOARD_NAME})
    elseif(TARGET MxGui::${BOARD_NAME})
        target_link_libraries(${TARGET} ${TYPE} MxGui::${BOARD_NAME})
    endif()
endfunction()


set(ASPIS_PASSES_PATH /home/streben/Documenti/ASPIS/build/passes)

# Given two libs, it creates a static library
function(apply_aspis_passes TO_PROTECT_LIB EXTERNAL_LIB OUTLIBRARY CXXFLAGS_RET)
    # Getting the sources of the two libraries
    get_property(TO_PROTECT_SRC TARGET ${TO_PROTECT_LIB} PROPERTY SOURCES)
    get_property(EXTERNAL_SRC TARGET ${EXTERNAL_LIB} PROPERTY SOURCES)

    # Create list of compiled protected sources for each board
    set(PROTECT_COMPILED_SOURCES TO_PROTECT_COMPILED)
    list(TRANSFORM TO_PROTECT_SRC APPEND ".o" OUTPUT_VARIABLE PROTECT_COMPILED_SOURCES)
    list(TRANSFORM PROTECT_COMPILED_SOURCES REPLACE ${CMAKE_SOURCE_DIR} "")
    list(TRANSFORM PROTECT_COMPILED_SOURCES PREPEND ${CMAKE_CURRENT_BINARY_DIR}/CMakeFiles/${TO_PROTECT_LIB}.dir)
    
    # Create list of compiled boardcore sources for each board
    set(EXTERNAL_COMPILED_SOURCES EXTERNAL_COMPILED)
    list(TRANSFORM EXTERNAL_SRC APPEND ".o" OUTPUT_VARIABLE EXTERNAL_COMPILED_SOURCES)
    list(TRANSFORM EXTERNAL_COMPILED_SOURCES REPLACE ${CMAKE_SOURCE_DIR} "")
    list(TRANSFORM EXTERNAL_COMPILED_SOURCES PREPEND ${CMAKE_CURRENT_BINARY_DIR}/CMakeFiles/${EXTERNAL_LIB}.dir)

    add_custom_target(
        ${OUTLIBRARY}-target
        BYPRODUCTS lib${OUTLIBRARY}.a
        COMMAND export LLVM_SYMBOLIZER_PATH=${LLVM_PREFIX}/llvm-symbolizer
        COMMAND ${MIOSIX_PREFIX}-link ${EXTERNAL_COMPILED_SOURCES} -o externals.bc
        COMMAND ${CMAKE_OPT} -load-pass-plugin=${ASPIS_PASSES_PATH}/libMarkToExclude.so --passes="mark-to-exclude" externals.bc -o externals.bc
        COMMAND ${MIOSIX_PREFIX}-link -v ${PROTECT_COMPILED_SOURCES} externals.bc -o out.bc
        # COMMAND ${CMAKE_OPT} -load-pass-plugin=${ASPIS_PASSES_PATH}/libEDDI.so --passes="func-ret-to-ref" out.bc -o ret_to_ref.bc
        # COMMAND ${CMAKE_OPT} -load-pass-plugin=${ASPIS_PASSES_PATH}/libSEDDI.so --passes="eddi-verify" ret_to_ref.bc -o out.bc -debug
        # COMMAND ${CMAKE_OPT} --passes="dce,simplifycfg" out.bc -o out.bc
        # COMMAND ${CMAKE_OPT} -load-pass-plugin=${ASPIS_PASSES_PATH}/libRASM.so --passes="rasm-verify" out.bc -o out.bc
        # COMMAND clang++ -c -v --target=arm-none-eabi ${CXXFLAGS_RET} -o compiled.o out.bc 
        COMMAND ${CMAKE_AR} ru lib${OUTLIBRARY}.a out.bc
        COMMAND_EXPAND_LISTS
        COMMENT "Applying ASPIS passes and creating archive lib${OUTLIBRARY}.a"
    )
    add_dependencies(${OUTLIBRARY}-target ${TO_PROTECT_LIB})
    add_dependencies(${OUTLIBRARY}-target ${EXTERNAL_LIB})

    # Importing the just built library
    add_library(${OUTLIBRARY} STATIC IMPORTED)
    add_dependencies(${OUTLIBRARY} ${OUTLIBRARY}-target)
    set_target_properties(${OUTLIBRARY} PROPERTIES IMPORTED_LOCATION ${CMAKE_CURRENT_BINARY_DIR}/lib${OUTLIBRARY}.a)
endfunction()


# Given two libs, it creates an executable
function(apply_aspis_passes_exe TO_PROTECT_EXE EXTERNAL_EXE OUTEXECUTABLE CXXFLAGS_RET)
    # Getting the sources of the two libraries
    get_property(TO_PROTECT_SRC TARGET ${TO_PROTECT_EXE} PROPERTY SOURCES)
    get_property(EXTERNAL_SRC TARGET ${EXTERNAL_EXE} PROPERTY SOURCES)

    # Create list of compiled protected sources for each board
    set(PROTECT_COMPILED_SOURCES TO_PROTECT_COMPILED)
    list(TRANSFORM TO_PROTECT_SRC APPEND ".o" OUTPUT_VARIABLE PROTECT_COMPILED_SOURCES)
    list(TRANSFORM PROTECT_COMPILED_SOURCES REPLACE ${CMAKE_SOURCE_DIR} "")
    list(TRANSFORM PROTECT_COMPILED_SOURCES PREPEND ${CMAKE_CURRENT_BINARY_DIR}/CMakeFiles/${TO_PROTECT_EXE}.dir/)
    
    # Create list of compiled boardcore sources for each board
    set(EXTERNAL_COMPILED_SOURCES EXTERNAL_COMPILED)
    list(TRANSFORM EXTERNAL_SRC APPEND ".o" OUTPUT_VARIABLE EXTERNAL_COMPILED_SOURCES)
    list(TRANSFORM EXTERNAL_COMPILED_SOURCES REPLACE ${CMAKE_SOURCE_DIR} "")
    list(TRANSFORM EXTERNAL_COMPILED_SOURCES PREPEND ${CMAKE_CURRENT_BINARY_DIR}/CMakeFiles/${EXTERNAL_EXE}.dir/)
    
    add_custom_target(
        ${OUTEXECUTABLE}-target
        BYPRODUCTS ${OUTEXECUTABLE}.elf
        COMMAND export LLVM_SYMBOLIZER_PATH=${LLVM_PREFIX}/llvm-symbolizer
        COMMAND ${MIOSIX_PREFIX}-link ${EXTERNAL_COMPILED_SOURCES} -o externals.bc
        COMMAND ${CMAKE_OPT} -load-pass-plugin=${ASPIS_PASSES_PATH}/libMarkToExclude.so --passes="mark-to-exclude" externals.bc -o externals.bc
        COMMAND ${MIOSIX_PREFIX}-link -v ${CMAKE_BINARY_DIR}/skyward_boardcore.bc ${PROTECT_COMPILED_SOURCES} externals.bc -o out.bc
        COMMAND ${CMAKE_OPT} -load-pass-plugin=${ASPIS_PASSES_PATH}/libEDDI.so --passes="func-ret-to-ref" out.bc -o ret_to_ref.bc
        COMMAND ${CMAKE_OPT} -load-pass-plugin=${ASPIS_PASSES_PATH}/libSEDDI.so --passes="eddi-verify" ret_to_ref.bc -o out.bc -debug
        COMMAND ${CMAKE_OPT} --passes="dce,simplifycfg" out.bc -o out.bc
        # COMMAND ${CMAKE_OPT} -load-pass-plugin=${ASPIS_PASSES_PATH}/libRASM.so --passes="rasm-verify" out.bc -o out.bc
        # COMMAND clang++ -v --target=arm-none-eabi ${CXXFLAGS_RET} -o ${OUTEXECUTABLE}.elf out.bc
        COMMAND clang++ --target=arm-none-eabi --sysroot=/opt/arm-miosix-eabi/arm-miosix-eabi/lib -DDEBUG -DEIGEN_MAX_ALIGN_BYTES=0 -DFMT_HEADER_ONLY=1 -DFMT_STATIC_THOUSANDS_SEPARATOR=0 -DFMT_UNICODE=0 -D_GLIBCXX_USE_WCHAR_T -D_MIOSIX -D_MIOSIX_GCC_PATCH_MAJOR=3 -D_MIOSIX_GCC_PATCH_MINOR=1 -D__GXX_TYPEINFO_EQUALITY_INLINE=0 -I/home/streben/Documenti/POLIMI/TESI/obsw/src -I/home/streben/Documenti/POLIMI/TESI/obsw/src/boards -I/home/streben/Documenti/POLIMI/TESI/obsw/skyward-boardcore/src/shared -I/home/streben/Documenti/POLIMI/TESI/obsw/skyward-boardcore/libs/miosix-kernel/miosix -I/home/streben/Documenti/POLIMI/TESI/obsw/skyward-boardcore/libs/miosix-kernel/miosix/arch/common -I/home/streben/Documenti/POLIMI/TESI/obsw/skyward-boardcore/libs/miosix-kernel/miosix/arch/cortexM7_stm32f7/common -I/home/streben/Documenti/POLIMI/TESI/obsw/skyward-boardcore/src/bsps/stm32f767zi_gemini_motor -I/home/streben/Documenti/POLIMI/TESI/obsw/skyward-boardcore/src/bsps/stm32f767zi_gemini_motor/config -I/home/streben/Documenti/POLIMI/TESI/obsw/skyward-boardcore/libs/miosix-kernel/miosix/default -I/opt/arm-miosix-eabi/arm-miosix-eabi/include/c++/9.2.0/arm-miosix-eabi -I/opt/arm-miosix-eabi/arm-miosix-eabi/include/c++/9.2.0 -I/opt/arm-miosix-eabi/arm-miosix-eabi/include -I/home/streben/Documenti/POLIMI/TESI/obsw/skyward-boardcore/libs/tscpp -I/home/streben/Documenti/POLIMI/TESI/obsw/skyward-boardcore/libs/eigen -I/home/streben/Documenti/POLIMI/TESI/obsw/skyward-boardcore/libs/fmt/include -I/home/streben/Documenti/POLIMI/TESI/obsw/skyward-boardcore/libs/Catch2/single_include -I/home/streben/Documenti/POLIMI/TESI/obsw/skyward-boardcore/libs/mavlink-skyward-lib -g -O0 -std=c++11 -D_MIOSIX_BOARDNAME=\"stm32f767zi_gemini_motor\" -D_DEFAULT_SOURCE=1 -std=c++14 -ffunction-sections -Wall -Werror=return-type -g -D_BOARD_STM32F767ZI_GEMINI_MOTOR -D_ARCH_CORTEXM7_STM32F7 -DHSE_VALUE=25000000 -DSYSCLK_FREQ_216MHz=216000000 -D__ENABLE_XRAM -mcpu=cortex-m7 -mthumb -mfloat-abi=hard -mfpu=fpv5-d16 -Wl,--gc-sections -Wl,-T/home/streben/Documenti/POLIMI/TESI/obsw/skyward-boardcore/src/bsps/stm32f767zi_gemini_motor/stm32_2m+16m_xram.ld -L/opt/arm-miosix-eabi/arm-miosix-eabi/lib/thumb/cm7/hardfp/fpv5 -L/opt/arm-miosix-eabi/lib/gcc/arm-miosix-eabi/9.2.0/thumb/cm7/hardfp/fpv5  -Wl,--start-group libmiosix.a -lstdc++ -lc -lm -lgcc -latomic -Wl,--end-group -nostdlib -fno-math-errno -MD -MT out.o -MF out.o.d  -o ${OUTEXECUTABLE}.elf out.bc /home/streben/Documenti/POLIMI/TESI/obsw/skyward-boardcore/libs/tscpp/tscpp/buffer.cpp /home/streben/Documenti/POLIMI/TESI/obsw/skyward-boardcore/libs/tscpp/tscpp/stream.cpp
        COMMAND_EXPAND_LISTS
        COMMENT "Applying ASPIS passes and creating ${OUTEXECUTABLE}.elf"
    )
    add_dependencies(${OUTEXECUTABLE}-target ${TO_PROTECT_EXE})
    add_dependencies(${OUTEXECUTABLE}-target ${EXTERNAL_EXE})

# /usr/lib64/ccache/clang++ --target=arm-none-eabi --sysroot=/opt/arm-miosix-eabi/arm-miosix-eabi/lib -g -O0 -Wl,-Map,/home/streben/Documenti/POLIMI/TESI/obsw/src/entrypoints/Main/build/main-entry.map -Wl,-Map,/home/streben/Documenti/POLIMI/TESI/obsw/src/entrypoints/Main/build/skyward-boardcore.map -mcpu=cortex-m7 -mthumb -mfloat-abi=hard -mfpu=fpv5-d16 -Wl,--gc-sections -Wl,-T/home/streben/Documenti/POLIMI/TESI/obsw/skyward-boardcore/src/bsps/stm32f767zi_gemini_motor/stm32_2m+16m_xram.ld -nostdlib -o main-entry.elf   -L/opt/arm-miosix-eabi/arm-miosix-eabi/lib/thumb/cm7/hardfp/fpv5  -L/opt/arm-miosix-eabi/lib/gcc/arm-miosix-eabi/9.2.0/thumb/cm7/hardfp/fpv5  libskyward-boardcore.a -Wl,--start-group libmiosix.a -lstdc++ -lc -lm -lgcc -latomic -Wl,--end-group


    # Importing the just built library
    add_executable(${OUTEXECUTABLE} IMPORTED)
    add_dependencies(${OUTEXECUTABLE} ${OUTEXECUTABLE}-target)
    set_target_properties(${OUTEXECUTABLE} PROPERTIES IMPORTED_LOCATION ${CMAKE_CURRENT_BINARY_DIR}/${OUTEXECUTABLE}.elf)
endfunction()

# COMMAND clang++ -v --target=arm-none-eabi --sysroot=/opt/arm-miosix-eabi/arm-miosix-eabi/lib -mcpu=cortex-m7 -mthumb -mfloat-abi=hard -mfpu=fpv5-d16 -Wl,--gc-sections -Wl,-T/home/streben/Documenti/POLIMI/TESI/obsw/skyward-boardcore/src/bsps/stm32f767zi_gemini_motor/stm32_2m+16m_xram.ld -nostdlib  -o ${OUTEXECUTABLE}.elf out.bc -L/opt/arm-miosix-eabi/arm-miosix-eabi/lib/thumb/cm7/hardfp/fpv5  -L/opt/arm-miosix-eabi/lib/gcc/arm-miosix-eabi/9.2.0/thumb/cm7/hardfp/fpv5

# Function to link the Miosix libraries to a target and register the build command
#
#   boardcore_link_target(<target>)
#
# What it does:
# - Links the Miosix libraries to the target
# - Tells the linker to generate the map file
# - Registers custom targets to create the hex and bin files (${TARGET}_bin and ${TARGET}_hex)
# - Registers a custom target to flash the program to the board (${TARGET}_program)
function(boardcore_link_target TARGET TYPE)

    # Linker script and linking options are eredited from miosix libraries

    # Linking skyward-boardcore library
    target_link_libraries(${TARGET} ${TYPE} skyward-boardcore)

    # Tell the linker to produce the map file
    target_link_options(${TARGET} ${TYPE} -Wl,-Map,$<TARGET_FILE_DIR:${TARGET}>/$<TARGET_FILE_BASE_NAME:${TARGET}>.map)  
    
    # # Add a post build command to create the hex file to flash on the board
    # add_custom_command(
    #     OUTPUT ${TARGET}.hex
    #     COMMAND ${CMAKE_OBJCOPY} -O ihex $<TARGET_FILE:${TARGET}> ${TARGET}.hex
    #     COMMENT "Creating ${TARGET}.hex"
    #     VERBATIM
    # )
    # add_custom_target(${TARGET}_hex ALL DEPENDS ${TARGET}.hex)
    # add_custom_command(
    #     OUTPUT ${TARGET}.bin
    #     COMMAND ${CMAKE_OBJCOPY} -O binary $<TARGET_FILE:${TARGET}> ${TARGET}.bin
    #     COMMENT "Creating ${TARGET}.bin"
    #     VERBATIM
    # )
    # add_custom_target(${TARGET}_bin ALL DEPENDS ${TARGET}.bin)

    # # Generate custom build command to flash the target
    # miosix_add_program_target(${TARGET})
endfunction()

